require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2'
require 'rexml/document'

include Amrita2
include Amrita2::Core

module Amrita2TableTest

  class TestTable < Test::Unit::TestCase
    include Amrita2
    include Amrita2::HtmlTag

    def test_match_many
      doc = REXML::Document.new <<-END
      <ul><li>1:<a href=''></a></li><li class='x'>2:<a href=''></a></li></ul>
      END
      #doc = REXML::Document.new <<-END
      #111<ul>222<li>333:<a href=''>444</a>555</li>666</ul>7777
      #END

      spec = Amrita2::define_template_spec(:direct_define=>true) {
        list(:matcher=>UL) {
          item(:matcher=>LI, :match_many=>true) {
            link(:matcher=>A, :replace_attr=>true)
          }
        }
      }
      t = spec.create_template(doc)
      out = ""
      t.expand_template(out) do |m|
        m.list do |ml|
          ml.item do |mi|
            mi.link('aaa', :href=>'bbb')
          end
        end
      end
      assert_equal("<ul><li>1:<a href='bbb'>aaa</a></li></ul>", out.strip)

      out = ""
      t.expand_template(out) do |m|
        m.list do |ml|
          ml.item(1) do |mi|
            mi.link('aaa', :href=>'bbb')
          end
        end
      end
      assert_equal("<ul><li class='x'>2:<a href='bbb'>aaa</a></li></ul>", out.strip)

      out = ""
      t.expand_template(out) do |m|
        m.list do |ml|
          ml.item(0) do |mi|
            mi.link('a', :href=>'b')
          end
          ml.item(1) do |mi|
            mi.link('c', :href=>'d')
          end
        end
      end
      assert_equal("<ul><li>1:<a href='b'>a</a></li><li class='x'>2:<a href='d'>c</a></li></ul>", out.strip)

    end

    def test_expand_table
      doc = REXML::Document.new <<-END
      <html>
      <body>
        <table border='1'>
          <thead>
            <tr id='header'><th>aaa</th><th>bbb</th></tr>
          </thead>
          <tbody>
            <tr class='odd'><td></td><td></td></tr>
            <tr class='even'><td></td><td></td></tr>
          </tbody>
        </table>
      </body>
      </html>
      END

      spec = Amrita2::define_template_spec(:direct_define=>true) {
        table(:matcher=>TABLE) {
          header
          detail(:delete_id=>true, :matcher=>TR, :match_many=>true) {
            aaa(:matcher=>TD)
            bbb(:matcher=>TD)
          }
        }
      }
      t = spec.create_template(doc)
      out = ""
      t.expand_template(out) do |m|
        m.table do |m2|
          5.times do |i|
            m2.detail(i) do |m3|
              m3.aaa('a' + i.to_s)
              m3.bbb('b' + i.to_s)
            end
          end
        end
      end
      assert_equal(<<-END.strip, out.strip)
<html>
      <body>
        <table border='1'>
          <thead>
            
          </thead>
          <tbody>
            <tr class='odd'><td>a0</td><td>b0</td></tr><tr class='even'><td>a1</td><td>b1</td></tr><tr class='odd'><td>a2</td><td>b2</td></tr><tr class='even'><td>a3</td><td>b3</td></tr><tr class='odd'><td>a4</td><td>b4</td></tr>
            
          </tbody>
        </table>
      </body>
      </html>
END

    end
  end
end

if __FILE__ == $0
  include Amrita2TableTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestTable.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end
